/**
 * Common types, jQuery API, Form and Data controllers, utility functions.
 * @module data/common
 */
/// <reference types="svelte" />
import type { Writable } from "svelte/store";
export declare const SVELTE_DATA_STORE: string;
export declare const SVELTE_FORM_STORE: string;
/** Return type for a simple svelte action; with destroy(), but without update(). */
export type ActionReturnType = {
    destroy: () => void;
} | void;
/** Format function, must return null if it can't parse value and doesn't want to override it. */
export interface Formatter {
    format: (val: DataTypes) => string;
    parse?: (val: string) => DataTypes | undefined;
}
/** Validation rule object: rule string and custom error prompt */
export type RuleObj = {
    type: string;
    prompt?: string;
};
/** Rule definition takes array or single instance of string or RuleObj */
export type RuleDefinition = string | string[] | RuleObj | RuleObj[];
/** Small subset of jQuery functions used in this library. */
export interface JQueryApi {
    parent(): JQueryApi;
    filter(selector: string): JQueryApi;
    find(selector: string): JQueryApi;
    prev(selector: string): JQueryApi;
    remove(): void;
    prop(name: string): string;
    attr(name: string): string;
    attr(name: string, value: string): void;
    hasClass(className: string): boolean;
    removeClass(className: string): void;
    length: number;
    on(event: string, selector: string | null, handler: () => void): void;
    off(event: string, selector: string | null, handler: () => void): void;
    data(key: string, value: unknown): void;
    data(key: string): unknown;
    removeData(key: string): void;
    each(fn: (idx: number, elem: Element) => void): void;
    text(): string;
    html(val: string): void;
    append(html: string): void;
    get(inx: number): Element;
    val(): string;
    val(val: string): void;
}
/** Gets jQuery element by id attribute. */
export declare function jQueryElemById(id: string): JQueryApi;
/** Gets jQuery element by dom node. */
export declare function jQueryElem(node: Element): JQueryApi;
/** Possible data types for the store */
export type DataTypes = string | string[] | boolean | Date | number | undefined;
/** Controls Semantic UI form element and it's data validation. */
export interface FormController {
    uid: string;
    mode: "sui-form" | "yup-form";
    valid: Writable<boolean>;
    errors: Writable<string[]>;
    getActive(): boolean;
    setActive(val: boolean): void;
    addRule: (key: string, rules: RuleDefinition) => void;
    doValidateField: (key: string) => void;
    doValidateForm: () => void;
    onFieldChange: (key: string) => void;
}
/** Holds Svelte `store` allowing the `Data` component to subscribe to data changes */
export interface DataController<T extends DataTypes> {
    /** Is used for debugging. */
    uid: string;
    /** Describes the type of the controlled component. */
    mode: "dropdown" | "modal" | "calendar" | "slider" | "input";
    /** Svelte store */
    store: Writable<T>;
    /** Imperatively updates Semantic component. */
    doUpdate: (val: T) => void;
    /** Called after user data entry or interaction. */
    onChange: (value: T) => void;
}
/** Semantic UI component behaviour API */
export type SemanticCommand = (command: string, v1?: unknown, v2?: unknown, v3?: unknown) => unknown;
/** Generate an unique number */
export declare function uid(): string;
/** Compare two arrays, two Date objects, or two primitives */
export declare function equalDataTypes(a1: DataTypes | undefined, a2: DataTypes | undefined): boolean;
export declare function pad(n: number, size: number): string;
export declare function isoDate(d: Date | undefined): string;
export declare function isoTime(d: Date | undefined): string;
